@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.867_01';
+our $VERSION = '1.839';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -26,15 +26,17 @@ $Carp::Internal{ (__PACKAGE__) }++;
 my ( $freeze, $thaw );
 
 BEGIN {
-   if ( $] ge '5.008008' && ! $INC{'PDL.pm'} ) {
-      local $@;
-      eval 'use Sereal::Encoder 3.015; use Sereal::Decoder 3.015;';
+   if ( ! defined $INC{'PDL.pm'} ) {
+      local $@; eval '
+         use Sereal::Encoder 3.015 qw( encode_sereal );
+         use Sereal::Decoder 3.015 qw( decode_sereal );
+      ';
       if ( ! $@ ) {
          my $encoder_ver = int( Sereal::Encoder->VERSION() );
          my $decoder_ver = int( Sereal::Decoder->VERSION() );
          if ( $encoder_ver - $decoder_ver == 0 ) {
-            $freeze = \&Sereal::Encoder::encode_sereal;
-            $thaw   = \&Sereal::Decoder::decode_sereal;
+            $freeze = \&encode_sereal;
+            $thaw   = \&decode_sereal;
          }
       }
    }
@@ -47,18 +49,19 @@ BEGIN {
 }
 
 use MCE::Util ();
+use bytes;
 
 my $is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
-my $tid = $INC{'threads.pm'} ? threads->tid() : 0;
+my $has_threads = $INC{'threads.pm'} ? 1 : 0;
+my $tid = $has_threads ? threads->tid() : 0;
 
 sub new {
    my ( $class, %argv ) = @_;
    my $impl = defined( $argv{impl} ) ? ucfirst( lc $argv{impl} ) : 'Mutex';
 
-   $impl = 'Threads' if ( $impl eq 'Mutex'   && $^O eq 'MSWin32' );
-   $impl = 'Mutex'   if ( $impl eq 'Threads' && $^O eq 'cygwin' );
+   $impl = 'Threads' if ( $^O eq 'MSWin32' && $impl eq 'Mutex' );
 
-   eval "require MCE::Channel::$impl; 1;" ||
+   eval "require MCE::Channel::$impl; 1" ||
       Carp::croak("Could not load Channel implementation '$impl': $@");
 
    my $pkg = 'MCE::Channel::'.$impl;
@@ -68,15 +71,16 @@ sub new {
 }
 
 sub CLONE {
-   $tid = threads->tid if $INC{'threads.pm'};
+   $tid = threads->tid if $has_threads;
 }
 
 sub DESTROY {
-   my ( $pid, $self ) = ( $tid ? $$ .'.'. $tid : $$, @_ );
+   my ( $pid, $self ) = ( $has_threads ? $$ .'.'. $tid : $$, @_ );
 
    if ( $self->{'init_pid'} && $self->{'init_pid'} eq $pid ) {
-      MCE::Util::_destroy_socks($self, qw(c_sock c2_sock p_sock p2_sock));
-      delete($self->{c_mutex}), delete($self->{p_mutex});
+      MCE::Util::_destroy_socks( $self, qw(c_sock p_sock) );
+      delete $self->{c_mutex};
+      delete $self->{p_mutex};
    }
 
    return;
@@ -108,7 +112,7 @@ sub _read {
 }
 
 sub _pid {
-   $tid ? $$ .'.'. $tid : $$;
+   $has_threads ? $$ .'.'. $tid : $$;
 }
 
 1;
@@ -127,7 +131,7 @@ MCE::Channel - Queue-like and two-way communication capability
 
 =head1 VERSION
 
-This document describes MCE::Channel version 1.867_01
+This document describes MCE::Channel version 1.839
 
 =head1 SYNOPSIS
 
@@ -209,23 +213,20 @@ socketpair handles. Serialization is provided by L<Sereal> if available.
 Defaults to L<Storable> otherwise. Excluding the C<Simple> implementation,
 both ends of the C<channel> support many workers concurrently (with mp => 1).
 
-=head2 new ( impl => STRING, mp => BOOLEAN )
+=head2 MCE::Channel->new ( impl => STRING, mp => BOOLEAN )
 
-This creates a new channel. Three implementations are provided C<Mutex>,
-C<Threads>, and C<Simple> indicating the locking mechanism to use
-C<MCE::Mutex>, C<threads::shared>, and no locking respectively.
+This creates a new channel. Three implementations are provided C<Mutex> (default),
+C<Threads>, and C<Simple> indicating the locking mechanism to use C<MCE::Mutex>,
+C<threads::shared>, and no locking respectively.
 
  $chnl = MCE::Channel->new();     # default: impl => 'Mutex', mp => 0
-                                  # default: impl => 'Threads' on Windows
 
-The C<Mutex> implementation supports processes and threads whereas the
-C<Threads> implementation is suited for Windows and threads only.
+The C<Mutex> channel implementation supports processes and threads whereas the
+C<Threads> channel implementation is suited for threads only.
 
  $chnl = MCE::Channel->new( impl => 'Mutex' );    # MCE::Mutex locking
  $chnl = MCE::Channel->new( impl => 'Threads' );  # threads::shared locking
 
- # on Windows, silently becomes impl => 'Threads' when specifying 'Mutex'
-
 Set the C<mp> (m)any (p)roducers option to a true value if there will be two
 or more workers calling C<enqueue>, <send>, C<recv2>, or C<recv2_nb> on the
 left end of the channel. This is important to not incur a race condition.
@@ -233,8 +234,6 @@ left end of the channel. This is important to not incur a race condition.
  $chnl = MCE::Channel->new( impl => 'Mutex', mp => 1 );
  $chnl = MCE::Channel->new( impl => 'Threads', mp => 1 );
 
- # on Windows, silently becomes impl => 'Threads' when specifying 'Mutex'
-
 The C<Simple> implementation is optimized for one producer and one consumer max.
 It omits locking for maximum performance. This implementation is preferred for
 parent to child communication not shared by another worker.
@@ -261,7 +260,7 @@ or defaults to L<Storable> otherwise.
  $chnl->enqueue({ hash_ref1 });
  $chnl->enqueue({ hash_ref2 }, { hash_ref3 }, ...);
 
-=head2 dequeue
+=head2 dequeue ()
 
 =head2 dequeue ( COUNT )
 
@@ -272,7 +271,7 @@ the method will block (i.e. until other producer(s) enqueue more items).
  $item  = $chnl->dequeue();       # item1
  @items = $chnl->dequeue(2);      # ( item2, item3 )
 
-=head2 dequeue_nb
+=head2 dequeue_nb ()
 
 =head2 dequeue_nb ( COUNT )
 
@@ -285,7 +284,7 @@ C<undef> in scalar context.
  $item  = $chnl->dequeue_nb();    # array_ref1
  @items = $chnl->dequeue_nb(2);   # ( array_ref2, array_ref3 )
 
-=head2 end
+=head2 end ()
 
 This is called by a producer to signal that there is no more work to be sent.
 Once ended, no more items may be sent by the producer. Calling C<end> by
@@ -293,7 +292,7 @@ multiple producers is not supported.
 
  $chnl->end;
 
-=head1 TWO-WAY IPC - PRODUCER TO CONSUMER
+=head1 TWO-WAY COMMUNICATION - PRODUCER TO CONSUMER
 
 =head2 send ( ARG1 [, ARG2, ... ] )
 
@@ -309,9 +308,9 @@ Object (de)serialization is handled automatically.
  $chnl->send($id, [ list_ref ]);
  $chnl->send($id, { hash_ref });
 
-=head2 recv
+=head2 recv ()
 
-=head2 recv_nb
+=head2 recv_nb ()
 
 Blocking and non-blocking fetch methods from the right end of the channel.
 For the latter and when the channel is empty, returns C<an empty list> in
@@ -325,7 +324,7 @@ list context or C<undef> in scalar context.
  ($id, $array_ref) = $chnl->recv_nb();
  ($id, $hash_ref)  = $chnl->recv_nb();
 
-=head1 TWO-WAY IPC - CONSUMER TO PRODUCER
+=head1 TWO-WAY COMMUNICATION - CONSUMER TO PRODUCER
 
 =head2 send2 ( ARG1 [, ARG2, ... ] )
 
@@ -341,9 +340,9 @@ Object (de)serialization is handled automatically.
  $chnl->send2($id, [ list_ref ]);
  $chnl->send2($id, { hash_ref });
 
-=head2 recv2
+=head2 recv2 ()
 
-=head2 recv2_nb
+=head2 recv2_nb ()
 
 Blocking and non-blocking fetch methods from the left end of the channel.
 For the latter and when the channel is empty, returns C<an empty list> in
@@ -357,9 +356,13 @@ list context or C<undef> in scalar context.
  ($id, $array_ref) = $chnl->recv2_nb();
  ($id, $hash_ref)  = $chnl->recv2_nb();
 
-=head1 DEMONSTRATIONS
+=head1 EXAMPLES
 
-=head2 Example 1 - threads
+=over 3
+
+=item Example 1 - threads
+
+=back
 
 C<MCE::Channel> was made to work efficiently with L<threads>. The reason is from
 using L<threads::shared> for locking versus L<MCE::Mutex>.
@@ -374,13 +377,11 @@ using L<threads::shared> for locking versus L<MCE::Mutex>.
  my $num_consumers = 10;
 
  sub consumer {
-    my $count = 0;
-
     # receive items
+    my $count = 0;
     while ( my ($item1, $item2) = $queue->dequeue(2) ) {
        $count += 2;
     }
-
     # send result
     $queue->send2( threads->tid => $count );
  }
@@ -422,7 +423,11 @@ using L<threads::shared> for locking versus L<MCE::Mutex>.
  7932
  80000 total
 
-=head2 Example 2 - MCE::Child
+=over 3
+
+=item Example 2 - MCE::Child
+
+=back
 
 The following is similarly threads-like for Perl lacking threads support.
 It spawns processes instead, thus requires the C<Mutex> channel implementation
@@ -438,13 +443,11 @@ which is the default if omitted.
  my $num_consumers = 10;
 
  sub consumer {
-    my $count = 0;
-
     # receive items
+    my $count = 0;
     while ( my ($item1, $item2) = $queue->dequeue(2) ) {
        $count += 2;
     }
-
     # send result
     $queue->send2( MCE::Child->pid => $count );
  }
@@ -470,67 +473,11 @@ which is the default if omitted.
  print $results{$_}, "\n" for keys %results;
  print "$total total\n\n";
 
-=head2 Example 3 - Consumer requests item
+=over 3
 
-Like the previous example, but have the manager process await a notification
-from the consumer before inserting into the queue. This allows the producer
-to end the channel early (i.e. exit loop).
+=item Example 3 - Two producers
 
- use strict;
- use warnings;
-
- use MCE::Child;
- use MCE::Channel;
-
- my $queue = MCE::Channel->new( impl => 'Mutex' );
- my $num_consumers = 10;
-
- sub consumer {
-    # receive items
-    my $count = 0;
-
-    while () {
-       # Notify the manager process to send items. This allows the
-       # manager process to enqueue only when requested. The benefit
-       # is being able to end the channel immediately.
-
-       $queue->send2( MCE::Child->pid ); # channel is bi-directional
-
-       my ($item1, $item2) = $queue->dequeue(2);
-       last unless ( defined $item1 );   # channel ended
-
-       $count += 2;
-    }
-
-    # result
-    return ( MCE::Child->pid => $count );
- }
-
- MCE::Child->create('consumer') for 1 .. $num_consumers;
-
- ## producer
-
- for my $num (1 .. 40000) {
-    # Await worker notification before inserting (blocking).
-    my $consumer_pid = $queue->recv2;
-    $queue->enqueue($num, $num * 2);
- }
-
- $queue->end;
-
- my %results;
- my $total = 0;
-
- for my $child ( MCE::Child->list ) {
-    my ($id, $count) = $child->join;
-    $results{$id} = $count;
-    $total += $count;
- }
-
- print $results{$_}, "\n" for keys %results;
- print "$total total\n\n";
-
-=head2 Example 4 - Many producers
+=back
 
 Running with 2 or more producers requires setting the C<mp> option. Internally,
 this enables locking support for the left end of the channel. The C<mp> option
@@ -590,7 +537,9 @@ Here, using the MCE facility for gathering the final count.
 
  print "$total total\n\n";
 
-=head2 Example 5 - Many channels
+=over 3
+
+=item Example 4 - Request input
 
 This demonstration configures a channel per consumer. Plus, a common channel
 for consumers to request the next input item. The C<Simple> implementation is
@@ -599,6 +548,8 @@ C<$ready> channel. However, consumers do not incur reading and what is written
 is very small (i.e. atomic write is guaranteed by the OS). Thus, am safely
 choosing the C<Simple> implementation versus C<Mutex>.
 
+=back
+
  use strict;
  use warnings;
 
@@ -713,9 +664,9 @@ Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2019-2020 by Mario E. Roy
+Copyright (C) 2019 by Mario E. Roy
 
-MCE::Channel is released under the same license as Perl.
+MCE::Shared is released under the same license as Perl.
 
 See L<http://dev.perl.org/licenses/> for more information.
 
