@@ -11,12 +11,13 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.867_01';
+our $VERSION = '1.839';
 
 use base 'MCE::Channel';
 use MCE::Mutex ();
+use bytes;
 
-my $LF = "\012"; Internals::SvREADONLY($LF, 1);
+my $is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
 my $freeze     = MCE::Channel::_get_freeze();
 my $thaw       = MCE::Channel::_get_thaw();
 
@@ -34,16 +35,15 @@ sub new {
 
    bless \%obj, $class;
 
-   MCE::Mutex::Channel::_save_for_global_cleanup($obj{c_mutex});
-   MCE::Mutex::Channel::_save_for_global_cleanup($obj{p_mutex}) if $obj{mp};
+   if ( caller !~ /^MCE:?/ || caller(1) !~ /^MCE:?/ ) {
+      MCE::Mutex::Channel::_save_for_global_destruction($obj{c_mutex});
+      MCE::Mutex::Channel::_save_for_global_destruction($obj{p_mutex})
+         if $obj{mp};
+   }
 
    return \%obj;
 }
 
-END {
-   MCE::Child->finish('MCE') if $INC{'MCE/Child.pm'};
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Queue-like methods.
@@ -54,7 +54,7 @@ sub end {
    my ( $self ) = @_;
    return if $self->{ended};
 
-   local $\ = undef if (defined $\);
+   MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
    print { $self->{p_sock} } pack('i', -1);
 
    $self->{ended} = 1;
@@ -64,12 +64,18 @@ sub enqueue {
    my $self = shift;
    return MCE::Channel::_ended('enqueue') if $self->{ended};
 
-   local $\ = undef if (defined $\);
    my $p_mutex = $self->{p_mutex};
    $p_mutex->lock2 if $p_mutex;
 
+   MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
+
    while ( @_ ) {
-      my $data = $freeze->([ shift ]);
+      my $data;
+      if ( ref $_[0] || !defined $_[0] ) {
+         $data = $freeze->([ shift ]), $data .= '1';
+      } else {
+         $data = shift, $data .= '0';
+      }
       print { $self->{p_sock} } pack('i', length $data), $data;
    }
 
@@ -84,6 +90,7 @@ sub dequeue {
 
    if ( $count == 1 ) {
       ( my $c_mutex = $self->{c_mutex} )->lock;
+      MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
       MCE::Util::_sysread( $self->{c_sock}, my($plen), 4 );
 
       my $len = unpack('i', $plen);
@@ -95,12 +102,15 @@ sub dequeue {
       MCE::Channel::_read( $self->{c_sock}, my($data), $len );
       $c_mutex->unlock;
 
-      wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+      chop( $data )
+         ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+         : wantarray ? ( $data ) : $data;
    }
    else {
       my ( $plen, @ret );
 
       ( my $c_mutex = $self->{c_mutex} )->lock;
+      MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
 
       while ( $count-- ) {
          MCE::Util::_sysread( $self->{c_sock}, $plen, 4 );
@@ -112,7 +122,7 @@ sub dequeue {
          }
 
          MCE::Channel::_read( $self->{c_sock}, my($data), $len );
-         push @ret, @{ $thaw->($data) };
+         push @ret, chop($data) ? @{ $thaw->($data) } : $data;
       }
 
       $c_mutex->unlock;
@@ -130,7 +140,7 @@ sub dequeue_nb {
 
    while ( $count-- ) {
       MCE::Util::_nonblocking( $self->{c_sock}, 1 );
-      MCE::Util::_sysread( $self->{c_sock}, $plen, 4 );
+      sysread( $self->{c_sock}, $plen, 4 );
       MCE::Util::_nonblocking( $self->{c_sock}, 0 );
 
       my $len; $len = unpack('i', $plen) if $plen;
@@ -140,7 +150,7 @@ sub dequeue_nb {
       }
 
       MCE::Channel::_read( $self->{c_sock}, my($data), $len );
-      push @ret, @{ $thaw->($data) };
+      push @ret, chop($data) ? @{ $thaw->($data) } : $data;
    }
 
    $c_mutex->unlock;
@@ -158,13 +168,19 @@ sub send {
    my $self = shift;
    return MCE::Channel::_ended('send') if $self->{ended};
 
-   my $data = $freeze->([ @_ ]);
+   my $data;
+   if ( @_ > 1 || ref $_[0] || !defined $_[0] ) {
+      $data = $freeze->([ @_ ]), $data .= '1';
+   } else {
+      $data = $_[0], $data .= '0';
+   }
 
-   local $\ = undef if (defined $\);
    my $p_mutex = $self->{p_mutex};
    $p_mutex->lock2 if $p_mutex;
 
+   MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
    print { $self->{p_sock} } pack('i', length $data), $data;
+
    $p_mutex->unlock2 if $p_mutex;
 
    return 1;
@@ -174,6 +190,7 @@ sub recv {
    my ( $self ) = @_;
 
    ( my $c_mutex = $self->{c_mutex} )->lock;
+   MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
    MCE::Util::_sysread( $self->{c_sock}, my($plen), 4 );
 
    my $len = unpack('i', $plen);
@@ -185,7 +202,9 @@ sub recv {
    MCE::Channel::_read( $self->{c_sock}, my($data), $len );
    $c_mutex->unlock;
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 sub recv_nb {
@@ -193,7 +212,7 @@ sub recv_nb {
 
    ( my $c_mutex = $self->{c_mutex} )->lock;
    MCE::Util::_nonblocking( $self->{c_sock}, 1 );
-   MCE::Util::_sysread( $self->{c_sock}, my($plen), 4 );
+   sysread( $self->{c_sock}, my($plen), 4 );
    MCE::Util::_nonblocking( $self->{c_sock}, 0 );
 
    my $len; $len = unpack('i', $plen) if $plen;
@@ -206,7 +225,9 @@ sub recv_nb {
    MCE::Channel::_read( $self->{c_sock}, my($data), $len );
    $c_mutex->unlock;
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 ###############################################################################
@@ -217,20 +238,18 @@ sub recv_nb {
 
 sub send2 {
    my $self = shift;
-   my $data = $freeze->([ @_ ]);
 
-   local $\ = undef if (defined $\);
-   local $MCE::Signal::SIG;
-
-   {
-      local $MCE::Signal::IPC = 1;
-      ( my $c_mutex = $self->{c_mutex} )->lock2;
-
-      print { $self->{c_sock} } pack('i', length $data), $data;
-      $c_mutex->unlock2;
+   my $data;
+   if ( @_ > 1 || ref $_[0] || !defined $_[0] ) {
+      $data = $freeze->([ @_ ]), $data .= '1';
+   } else {
+      $data = $_[0], $data .= '0';
    }
 
-   CORE::kill($MCE::Signal::SIG, $$) if $MCE::Signal::SIG;
+   ( my $c_mutex = $self->{c_mutex} )->lock2;
+   MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
+   print { $self->{c_sock} } pack('i', length $data), $data;
+   $c_mutex->unlock2;
 
    return 1;
 }
@@ -239,37 +258,39 @@ sub recv2 {
    my ( $self ) = @_;
    my ( $plen, $data );
 
-   local $/ = $LF if ( $/ ne $LF );
    my $p_mutex = $self->{p_mutex};
    $p_mutex->lock if $p_mutex;
 
-   ( $p_mutex )
-      ? MCE::Util::_sysread( $self->{p_sock}, $plen, 4 )
+   MCE::Util::_sock_ready( $self->{p_sock} ) if $is_MSWin32;
+
+   ( $p_mutex || $is_MSWin32 )
+      ? sysread( $self->{p_sock}, $plen, 4 )
       : read( $self->{p_sock}, $plen, 4 );
 
    my $len = unpack('i', $plen);
 
-   ( $p_mutex )
+   ( $p_mutex || $is_MSWin32 )
       ? MCE::Channel::_read( $self->{p_sock}, $data, $len )
       : read( $self->{p_sock}, $data, $len );
 
    $p_mutex->unlock if $p_mutex;
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 sub recv2_nb {
    my ( $self ) = @_;
    my ( $plen, $data );
 
-   local $/ = $LF if ( $/ ne $LF );
    my $p_mutex = $self->{p_mutex};
    $p_mutex->lock if $p_mutex;
 
    MCE::Util::_nonblocking( $self->{p_sock}, 1 );
 
-   ( $p_mutex )
-      ? MCE::Util::_sysread( $self->{p_sock}, $plen, 4 )
+   ( $p_mutex || $is_MSWin32 )
+      ? sysread( $self->{p_sock}, $plen, 4 )
       : read( $self->{p_sock}, $plen, 4 );
 
    MCE::Util::_nonblocking( $self->{p_sock}, 0 );
@@ -280,13 +301,15 @@ sub recv2_nb {
       return wantarray ? () : undef;
    }
 
-   ( $p_mutex )
+   ( $p_mutex || $is_MSWin32 )
       ? MCE::Channel::_read( $self->{p_sock}, $data, $len )
       : read( $self->{p_sock}, $data, $len );
 
    $p_mutex->unlock if $p_mutex;
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 1;
@@ -305,69 +328,25 @@ MCE::Channel::Mutex - Channel for producer(s) and many consumers
 
 =head1 VERSION
 
-This document describes MCE::Channel::Mutex version 1.867_01
+This document describes MCE::Channel::Mutex version 1.839
 
 =head1 DESCRIPTION
 
 A channel class providing queue-like and two-way communication
 for processes and threads. Locking is handled using MCE::Mutex.
 
-The API is described in L<MCE::Channel>.
-
-=over 3
-
-=item new
-
  use MCE::Channel;
 
  # The default is tuned for one producer and many consumers.
- my $chnl_a = MCE::Channel->new( impl => 'Mutex' );
+ my $chnl_a = MCE::Channel->new( impl => 'mutex' );
 
  # Specify the 'mp' option for safe use by two or more producers
- # sending or recieving on the left side of the channel (i.e.
- # ->enqueue/->send or ->recv2/->recv2_nb).
+ # sending or recieving on the left side of the channel.
+ # E.g. C<->enqueue/->send> or C<->recv2/->recv2_nb>
 
- my $chnl_b = MCE::Channel->new( impl => 'Mutex', mp => 1 );
+ my $chnl_b = MCE::Channel->new( impl => 'mutex', mp => 1 );
 
-=back
-
-=head1 QUEUE-LIKE BEHAVIOR
-
-=over 3
-
-=item enqueue
-
-=item dequeue
-
-=item dequeue_nb
-
-=item end
-
-=back
-
-=head1 TWO-WAY IPC - PRODUCER TO CONSUMER
-
-=over 3
-
-=item send
-
-=item recv
-
-=item recv_nb
-
-=back
-
-=head1 TWO-WAY IPC - CONSUMER TO PRODUCER
-
-=over 3
-
-=item send2
-
-=item recv2
-
-=item recv2_nb
-
-=back
+The API is described in L<MCE::Channel>.
 
 =head1 AUTHOR
 
