@@ -11,11 +11,11 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.867_01';
+our $VERSION = '1.839';
 
 use base 'MCE::Channel';
+use bytes;
 
-my $LF = "\012"; Internals::SvREADONLY($LF, 1);
 my $is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
 my $freeze     = MCE::Channel::_get_freeze();
 my $thaw       = MCE::Channel::_get_thaw();
@@ -39,7 +39,6 @@ sub end {
    my ( $self ) = @_;
    return if $self->{ended};
 
-   local $\ = undef if (defined $\);
    MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
    print { $self->{p_sock} } pack('i', -1);
 
@@ -50,11 +49,15 @@ sub enqueue {
    my $self = shift;
    return MCE::Channel::_ended('enqueue') if $self->{ended};
 
-   local $\ = undef if (defined $\);
    MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
 
    while ( @_ ) {
-      my $data = $freeze->([ shift ]);
+      my $data;
+      if ( ref $_[0] || !defined $_[0] ) {
+         $data = $freeze->([ shift ]), $data .= '1';
+      } else {
+         $data = shift, $data .= '0';
+      }
       print { $self->{p_sock} } pack('i', length $data) . $data;
    }
 
@@ -65,10 +68,9 @@ sub dequeue {
    my ( $self, $count ) = @_;
    $count = 1 if ( !$count || $count < 1 );
 
-   local $/ = $LF if ( $/ ne $LF );
-
    if ( $count == 1 ) {
       my ( $plen, $data );
+
       MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
 
       $is_MSWin32
@@ -85,10 +87,13 @@ sub dequeue {
          ? MCE::Channel::_read( $self->{c_sock}, $data, $len )
          : read( $self->{c_sock}, $data, $len );
 
-      wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+      chop( $data )
+         ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+         : wantarray ? ( $data ) : $data;
    }
    else {
       my ( $plen, @ret );
+
       MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
 
       while ( $count-- ) {
@@ -108,7 +113,7 @@ sub dequeue {
             ? MCE::Channel::_read( $self->{c_sock}, $data, $len )
             : read( $self->{c_sock}, $data, $len );
 
-         push @ret, @{ $thaw->($data) };
+         push @ret, chop($data) ? @{ $thaw->($data) } : $data;
       }
 
       wantarray ? @ret : $ret[-1];
@@ -120,10 +125,10 @@ sub dequeue_nb {
    $count = 1 if ( !$count || $count < 1 );
 
    my ( $plen, @ret );
-   local $/ = $LF if ( $/ ne $LF );
 
    while ( $count-- ) {
       my $data;
+
       MCE::Util::_nonblocking( $self->{c_sock}, 1 );
 
       $is_MSWin32
@@ -142,7 +147,7 @@ sub dequeue_nb {
          ? MCE::Channel::_read( $self->{c_sock}, $data, $len )
          : read( $self->{c_sock}, $data, $len );
 
-      push @ret, @{ $thaw->($data) };
+      push @ret, chop($data) ? @{ $thaw->($data) } : $data;
    }
 
    wantarray ? @ret : $ret[-1];
@@ -158,9 +163,13 @@ sub send {
    my $self = shift;
    return MCE::Channel::_ended('send') if $self->{ended};
 
-   my $data = $freeze->([ @_ ]);
+   my $data;
+   if ( @_ > 1 || ref $_[0] || !defined $_[0] ) {
+      $data = $freeze->([ @_ ]), $data .= '1';
+   } else {
+      $data = $_[0], $data .= '0';
+   }
 
-   local $\ = undef if (defined $\);
    MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
    print { $self->{p_sock} } pack('i', length $data) . $data;
 
@@ -171,7 +180,6 @@ sub recv {
    my ( $self ) = @_;
    my ( $plen, $data );
 
-   local $/ = $LF if ( $/ ne $LF );
    MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
 
    $is_MSWin32
@@ -188,14 +196,15 @@ sub recv {
       ? MCE::Channel::_read( $self->{c_sock}, $data, $len )
       : read( $self->{c_sock}, $data, $len );
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 sub recv_nb {
    my ( $self ) = @_;
    my ( $plen, $data );
 
-   local $/ = $LF if ( $/ ne $LF );
    MCE::Util::_nonblocking( $self->{c_sock}, 1 );
 
    $is_MSWin32
@@ -214,7 +223,9 @@ sub recv_nb {
       ? MCE::Channel::_read( $self->{c_sock}, $data, $len )
       : read( $self->{c_sock}, $data, $len );
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 ###############################################################################
@@ -225,19 +236,16 @@ sub recv_nb {
 
 sub send2 {
    my $self = shift;
-   my $data = $freeze->([ @_ ]);
 
-   local $\ = undef if (defined $\);
-   local $MCE::Signal::SIG;
-
-   {
-      local $MCE::Signal::IPC = 1;
-
-      MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
-      print { $self->{c_sock} } pack('i', length $data) . $data;
+   my $data;
+   if ( @_ > 1 || ref $_[0] || !defined $_[0] ) {
+      $data = $freeze->([ @_ ]), $data .= '1';
+   } else {
+      $data = $_[0], $data .= '0';
    }
 
-   CORE::kill($MCE::Signal::SIG, $$) if $MCE::Signal::SIG;
+   MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
+   print { $self->{c_sock} } pack('i', length $data) . $data;
 
    return 1;
 }
@@ -246,7 +254,6 @@ sub recv2 {
    my ( $self ) = @_;
    my ( $plen, $data );
 
-   local $/ = $LF if ( $/ ne $LF );
    MCE::Util::_sock_ready( $self->{p_sock} ) if $is_MSWin32;
 
    $is_MSWin32
@@ -259,14 +266,15 @@ sub recv2 {
       ? MCE::Channel::_read( $self->{p_sock}, $data, $len )
       : read( $self->{p_sock}, $data, $len );
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 sub recv2_nb {
    my ( $self ) = @_;
    my ( $plen, $data );
 
-   local $/ = $LF if ( $/ ne $LF );
    MCE::Util::_nonblocking( $self->{p_sock}, 1 );
 
    $is_MSWin32
@@ -282,7 +290,9 @@ sub recv2_nb {
       ? MCE::Channel::_read( $self->{p_sock}, $data, $len )
       : read( $self->{p_sock}, $data, $len );
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 1;
@@ -301,62 +311,18 @@ MCE::Channel::Simple - Channel tuned for one producer and one consumer
 
 =head1 VERSION
 
-This document describes MCE::Channel::Simple version 1.867_01
+This document describes MCE::Channel::Simple version 1.839
 
 =head1 DESCRIPTION
 
 A channel class providing queue-like and two-way communication
 for one process or thread on either end; no locking needed.
 
-The API is described in L<MCE::Channel>.
-
-=over 3
-
-=item new
-
  use MCE::Channel;
 
- my $chnl = MCE::Channel->new( impl => 'Simple' );
+ my $chnl = MCE::Channel->new( impl => 'simple' );
 
-=back
-
-=head1 QUEUE-LIKE BEHAVIOR
-
-=over 3
-
-=item enqueue
-
-=item dequeue
-
-=item dequeue_nb
-
-=item end
-
-=back
-
-=head1 TWO-WAY IPC - PRODUCER TO CONSUMER
-
-=over 3
-
-=item send
-
-=item recv
-
-=item recv_nb
-
-=back
-
-=head1 TWO-WAY IPC - CONSUMER TO PRODUCER
-
-=over 3
-
-=item send2
-
-=item recv2
-
-=item recv2_nb
-
-=back
+The API is described in L<MCE::Channel>.
 
 =head1 AUTHOR
 
