@@ -11,14 +11,14 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.867_01';
+our $VERSION = '1.839';
 
 use threads;
 use threads::shared;
 
 use base 'MCE::Channel';
+use bytes;
 
-my $LF = "\012"; Internals::SvREADONLY($LF, 1);
 my $is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
 my $freeze     = MCE::Channel::_get_freeze();
 my $thaw       = MCE::Channel::_get_thaw();
@@ -28,7 +28,6 @@ sub new {
 
    $obj{init_pid} = MCE::Channel::_pid();
    MCE::Util::_sock_pair( \%obj, 'p_sock', 'c_sock' );
-   MCE::Util::_sock_pair( \%obj, 'p2_sock', 'c2_sock' ) if $is_MSWin32;
 
    # locking for the consumer side of the channel
    $obj{cr_mutex} = threads::shared::share( my $cr_mutex );
@@ -51,7 +50,6 @@ sub end {
    my ( $self ) = @_;
    return if $self->{ended};
 
-   local $\ = undef if (defined $\);
    MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
    print { $self->{p_sock} } pack('i', -1);
 
@@ -62,14 +60,17 @@ sub enqueue {
    my $self = shift;
    return MCE::Channel::_ended('enqueue') if $self->{ended};
 
-   local $\ = undef if (defined $\);
-
    {
       CORE::lock $self->{pw_mutex} if $self->{pw_mutex};
       MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
 
       while ( @_ ) {
-         my $data = $freeze->([ shift ]);
+         my $data;
+         if ( ref $_[0] || !defined $_[0] ) {
+            $data = $freeze->([ shift ]), $data .= '1';
+         } else {
+            $data = shift, $data .= '0';
+         }
          print { $self->{p_sock} } pack('i', length $data), $data;
       }
    }
@@ -98,7 +99,9 @@ sub dequeue {
          MCE::Channel::_read( $self->{c_sock}, $data, $len );
       }
 
-      wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+      chop( $data )
+         ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+         : wantarray ? ( $data ) : $data;
    }
    else {
       my ( $plen, @ret );
@@ -117,7 +120,7 @@ sub dequeue {
             }
 
             MCE::Channel::_read( $self->{c_sock}, my($data), $len );
-            push @ret, @{ $thaw->($data) };
+            push @ret, chop($data) ? @{ $thaw->($data) } : $data;
          }
       }
 
@@ -136,7 +139,7 @@ sub dequeue_nb {
 
       while ( $count-- ) {
          MCE::Util::_nonblocking( $self->{c_sock}, 1 );
-         MCE::Util::_sysread( $self->{c_sock}, $plen, 4 );
+         sysread( $self->{c_sock}, $plen, 4 );
          MCE::Util::_nonblocking( $self->{c_sock}, 0 );
 
          my $len; $len = unpack('i', $plen) if $plen;
@@ -146,7 +149,7 @@ sub dequeue_nb {
          }
 
          MCE::Channel::_read( $self->{c_sock}, my($data), $len );
-         push @ret, @{ $thaw->($data) };
+         push @ret, chop($data) ? @{ $thaw->($data) } : $data;
       }
    }
 
@@ -163,9 +166,12 @@ sub send {
    my $self = shift;
    return MCE::Channel::_ended('send') if $self->{ended};
 
-   my $data = $freeze->([ @_ ]);
-
-   local $\ = undef if (defined $\);
+   my $data;
+   if ( @_ > 1 || ref $_[0] || !defined $_[0] ) {
+      $data = $freeze->([ @_ ]), $data .= '1';
+   } else {
+      $data = $_[0], $data .= '0';
+   }
 
    {
       CORE::lock $self->{pw_mutex} if $self->{pw_mutex};
@@ -194,7 +200,9 @@ sub recv {
       MCE::Channel::_read( $self->{c_sock}, $data, $len );
    }
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 sub recv_nb {
@@ -204,7 +212,7 @@ sub recv_nb {
    {
       CORE::lock $self->{cr_mutex};
       MCE::Util::_nonblocking( $self->{c_sock}, 1 );
-      MCE::Util::_sysread( $self->{c_sock}, $plen, 4 );
+      sysread( $self->{c_sock}, $plen, 4 );
       MCE::Util::_nonblocking( $self->{c_sock}, 0 );
 
       my $len; $len = unpack('i', $plen) if $plen;
@@ -216,7 +224,9 @@ sub recv_nb {
       MCE::Channel::_read( $self->{c_sock}, $data, $len );
    }
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 ###############################################################################
@@ -227,22 +237,19 @@ sub recv_nb {
 
 sub send2 {
    my $self = shift;
-   my $data = $freeze->([ @_ ]);
 
-   local $\ = undef if (defined $\);
-   local $MCE::Signal::SIG;
-
-   {
-      my $c_sock = $self->{c2_sock} || $self->{c_sock};
-
-      local $MCE::Signal::IPC = 1;
-      CORE::lock $self->{cw_mutex};
-
-      MCE::Util::_sock_ready_w( $c_sock ) if $is_MSWin32;
-      print { $c_sock } pack('i', length $data), $data;
+   my $data;
+   if ( @_ > 1 || ref $_[0] || !defined $_[0] ) {
+      $data = $freeze->([ @_ ]), $data .= '1';
+   } else {
+      $data = $_[0], $data .= '0';
    }
 
-   CORE::kill($MCE::Signal::SIG, $$) if $MCE::Signal::SIG;
+   {
+      CORE::lock $self->{cw_mutex};
+      MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
+      print { $self->{c_sock} } pack('i', length $data), $data;
+   }
 
    return 1;
 }
@@ -251,58 +258,56 @@ sub recv2 {
    my ( $self ) = @_;
    my ( $plen, $data );
 
-   local $/ = $LF if ( $/ ne $LF );
-
    {
-      my $p_sock   = $self->{p2_sock} || $self->{p_sock};
       my $pr_mutex = $self->{pr_mutex};
-
       CORE::lock $pr_mutex if $pr_mutex;
-      MCE::Util::_sock_ready( $p_sock ) if $is_MSWin32;
+
+      MCE::Util::_sock_ready( $self->{p_sock} ) if $is_MSWin32;
 
       ( $pr_mutex || $is_MSWin32 )
-         ? MCE::Util::_sysread( $p_sock, $plen, 4 )
-         : read( $p_sock, $plen, 4 );
+         ? sysread( $self->{p_sock}, $plen, 4 )
+         : read( $self->{p_sock}, $plen, 4 );
 
       my $len = unpack('i', $plen);
 
       ( $pr_mutex || $is_MSWin32 )
-         ? MCE::Channel::_read( $p_sock, $data, $len )
-         : read( $p_sock, $data, $len );
+         ? MCE::Channel::_read( $self->{p_sock}, $data, $len )
+         : read( $self->{p_sock}, $data, $len );
    }
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 sub recv2_nb {
    my ( $self ) = @_;
    my ( $plen, $data );
 
-   local $/ = $LF if ( $/ ne $LF );
-
    {
-      my $p_sock   = $self->{p2_sock} || $self->{p_sock};
       my $pr_mutex = $self->{pr_mutex};
-
       CORE::lock $pr_mutex if $pr_mutex;
-      MCE::Util::_nonblocking( $p_sock, 1 );
+
+      MCE::Util::_nonblocking( $self->{p_sock}, 1 );
 
       ( $pr_mutex || $is_MSWin32 )
-         ? MCE::Util::_sysread( $p_sock, $plen, 4 )
-         : read( $p_sock, $plen, 4 );
+         ? sysread( $self->{p_sock}, $plen, 4 )
+         : read( $self->{p_sock}, $plen, 4 );
 
-      MCE::Util::_nonblocking( $p_sock, 0 );
+      MCE::Util::_nonblocking( $self->{p_sock}, 0 );
 
       my $len; $len = unpack('i', $plen) if $plen;
 
       return wantarray ? () : undef unless $len;
 
       ( $pr_mutex || $is_MSWin32 )
-         ? MCE::Channel::_read( $p_sock, $data, $len )
-         : read( $p_sock, $data, $len );
+         ? MCE::Channel::_read( $self->{p_sock}, $data, $len )
+         : read( $self->{p_sock}, $data, $len );
    }
 
-   wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1];
+   chop( $data )
+      ? wantarray ? @{ $thaw->($data) } : ( $thaw->($data) )->[-1]
+      : wantarray ? ( $data ) : $data;
 }
 
 1;
@@ -321,76 +326,25 @@ MCE::Channel::Threads - Channel for producer(s) and many consumers
 
 =head1 VERSION
 
-This document describes MCE::Channel::Threads version 1.867_01
+This document describes MCE::Channel::Threads version 1.839
 
 =head1 DESCRIPTION
 
 A channel class providing queue-like and two-way communication
-for threads only. Locking is handled using threads::shared.
-
-The API is described in L<MCE::Channel>.
-
-=over 3
-
-=item new
+for threads only. Locking is handled using threads.
 
  use MCE::Channel;
 
  # The default is tuned for one producer and many consumers.
- my $chnl_a = MCE::Channel->new( impl => 'Threads' );
+ my $chnl_a = MCE::Channel->new( impl => 'threads' );
 
  # Specify the 'mp' option for safe use by two or more producers
- # sending or recieving on the left side of the channel (i.e.
- # ->enqueue/->send or ->recv2/->recv2_nb).
+ # sending or recieving on the left side of the channel.
+ # E.g. C<->enqueue/->send> or C<->recv2/->recv2_nb>
 
- my $chnl_b = MCE::Channel->new( impl => 'Threads', mp => 1 );
+ my $chnl_b = MCE::Channel->new( impl => 'threads', mp => 1 );
 
-=back
-
-=head1 QUEUE-LIKE BEHAVIOR
-
-=over 3
-
-=item enqueue
-
-=item dequeue
-
-=item dequeue_nb
-
-=item end
-
-=back
-
-=head1 TWO-WAY IPC - PRODUCER TO CONSUMER
-
-=over 3
-
-=item send
-
-=item recv
-
-=item recv_nb
-
-=back
-
-=head1 TWO-WAY IPC - CONSUMER TO PRODUCER
-
-=over 3
-
-=item send2
-
-=item recv2
-
-=item recv2_nb
-
-=back
-
-=head1 LIMITATIONS
-
-The t/04_channel_threads tests are disabled on Unix platforms for Perl
-less than 5.10.1. Basically, the MCE::Channel::Threads implementation
-is not supported on older Perls unless the OS vendor applied upstream
-patches (i.e. works on RedHat/CentOS 5.x running Perl 5.8.x).
+The API is described in L<MCE::Channel>.
 
 =head1 AUTHOR
 
