@@ -11,7 +11,7 @@ no warnings qw( threads recursion uninitialized once redefine );
 
 package MCE::Child;
 
-our $VERSION = '1.867_01';
+our $VERSION = '1.839';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
@@ -19,9 +19,9 @@ our $VERSION = '1.867_01';
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use MCE::Signal ();
-use MCE::Mutex ();
-use MCE::Channel ();
+use MCE::Channel;
 use Time::HiRes 'sleep';
+use bytes;
 
 use overload (
    q(==)    => \&equal,
@@ -31,7 +31,7 @@ use overload (
 
 sub import {
    no strict 'refs'; no warnings 'redefine';
-   *{ caller().'::mce_child' } = \&mce_child;
+   *{ caller().'::mce_child' } = \&child;
    return;
 }
 
@@ -48,11 +48,12 @@ my ( $_MNGD, $_DATA, $_DELY, $_LIST ) = ( {}, {}, {}, {} );
 my $_freeze = MCE::Channel::_get_freeze();
 my $_thaw   = MCE::Channel::_get_thaw();
 
-my $_is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
-my $_tid        = ( $INC{'threads.pm'} ) ? threads->tid() : 0;
+my $_is_MSWin32  = ($^O eq 'MSWin32') ? 1 : 0;
+my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
+my $_tid = $_has_threads ? threads->tid() : 0;
 
 sub CLONE {
-   $_tid = threads->tid(), &_clear() if $INC{'threads.pm'};
+   $_tid = threads->tid(), &_clear() if $_has_threads;
 }
 
 sub _clear {
@@ -84,30 +85,26 @@ sub init {
 
    &_force_reap($pkg), $_DATA->{$pkg}->clear() if exists $_LIST->{$pkg};
 
+   # Start the shared-manager process if not running.
+   MCE::Shared->start() if $INC{'MCE/Shared.pm'};
+
    if ( !exists $_LIST->{$pkg} ) {
-      $MCE::_GMUTEX->lock() if ( $_tid && $MCE::_GMUTEX );
-      sleep 0.015 if $_tid;
-
-      # Start the shared-manager process if not running.
-      MCE::Shared->start() if $INC{'MCE/Shared.pm'};
-
       my $chnl = MCE::Channel->new( impl => 'Mutex' );
       $_LIST->{ $pkg } = MCE::Child::_ordhash->new();
       $_DELY->{ $pkg } = MCE::Child::_delay->new( $chnl );
       $_DATA->{ $pkg } = MCE::Child::_hash->new( $chnl );
       $_DATA->{"$pkg:seed"} = int(rand() * 1e9);
       $_DATA->{"$pkg:id"  } = 0;
-
-      $MCE::_GMUTEX->unlock() if ( $_tid && $MCE::_GMUTEX );
    }
 
    if ( !exists $mngd->{posix_exit} ) {
       $mngd->{posix_exit} = 1 if (
-         $^S || $_tid || $INC{'Mojo/IOLoop.pm'} ||
-         $INC{'Coro.pm'} || $INC{'LWP/UserAgent.pm'} || $INC{'stfl.pm'} ||
+         ( $_has_threads && $_tid ) || $INC{'Mojo/IOLoop.pm'} ||
          $INC{'Curses.pm'} || $INC{'CGI.pm'} || $INC{'FCGI.pm'} ||
-         $INC{'Tk.pm'} || $INC{'Wx.pm'} || $INC{'Win32/GUI.pm'} ||
-         $INC{'Gearman/Util.pm'} || $INC{'Gearman/XS.pm'}
+         $INC{'Prima.pm'} || $INC{'Tk.pm'} || $INC{'Wx.pm'} ||
+         $INC{'Gearman/Util.pm'} || $INC{'Gearman/XS.pm'} ||
+         $INC{'Coro.pm'} || $INC{'LWP/UserAgent.pm'} ||
+         $INC{'Win32/GUI.pm'} || $INC{'stfl.pm'}
       );
    }
 
@@ -130,7 +127,7 @@ sub init {
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## 'new', 'mce_child', and 'create' for threads-like similarity.
+## 'new', 'child (mce_child)', and 'create' for threads-like similarity.
 ##
 ###############################################################################
 
@@ -141,7 +138,7 @@ sub init {
 ## Use "goto" trick to avoid pad problems from 5.8.1 (fixed in 5.8.2)
 ## Tip found in threads::async.
 
-sub mce_child (&;@) {
+sub child (&;@) {
    goto &create;
 }
 
@@ -157,7 +154,6 @@ sub create {
 
    my $self = bless ref $_[0] eq 'HASH' ? { %{ shift() } } : { }, __PACKAGE__;
 
-   $self->{IGNORE} = 1 if $SIG{CHLD} eq 'IGNORE';
    $self->{MGR_ID} = $mngd->{MGR_ID}, $self->{PKG} = $mngd->{PKG};
    $self->{ident } = shift if ( !ref $_[0] && ref $_[1] eq 'CODE' );
 
@@ -179,10 +175,9 @@ sub create {
       local $!;
 
       # Reap completed child processes.
-      $_DATA->{$pkg}->reap_data;
+      $_DATA->{$pkg}->reapdata;
 
       for my $wrk_id ( keys %{ $list->[0] } ) {
-         $list->del($wrk_id), next if ( exists $list->[0]{$wrk_id}{JOINED} );
          waitpid($wrk_id, _WNOHANG) or next;
          _reap_child($list->del($wrk_id), 0);
       }
@@ -196,81 +191,66 @@ sub create {
 
    # ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~
 
-   $MCE::_GMUTEX->lock() if ( $_tid && $MCE::_GMUTEX );
+   local $SIG{TTIN}  unless $_is_MSWin32;
+   local $SIG{TTOU}  unless $_is_MSWin32;
+   local $SIG{WINCH} unless $_is_MSWin32;
 
    my @args = @_; @_ = ();  # To avoid (Scalars leaked: N) messages
-   my ( $killed, $pid );
+   my $pid  = fork();
 
-   {
-      local $SIG{TERM} = local $SIG{INT} = sub { $killed = $_[0] }
-         if ( !$_is_MSWin32 && $] ge '5.010001' );
+   if ( !defined $pid ) {                                # error
+      local $\; print {*STDERR} "fork error: $!\n";
 
-      local $SIG{TTIN}, local $SIG{TTOU}, local $SIG{WINCH}
-         if ( !$_is_MSWin32 );
+      return undef;
+   }
+   elsif ( $pid ) {                                      # parent
+      $self->{WRK_ID} = $pid, $list->set($pid, $self);
+      $mngd->{on_start}->($pid, $self->{ident}) if $mngd->{on_start};
 
-      $pid = fork();
-
-      if ( !defined $pid ) {                                # error
-         local $\; print {*STDERR} "fork error: $!\n";
-      }
-      elsif ( $pid ) {                                      # parent
-         $self->{WRK_ID} = $pid;
-         $list->set($pid, $self) unless $self->{IGNORE};
-         $mngd->{on_start}->($pid, $self->{ident}) if $mngd->{on_start};
-      }
-      else {                                                # child
-         %{ $_LIST } = (), $_SELF = $self;
-
-         local $SIG{TERM} = local $SIG{INT} = \&_trap,
-         local $SIG{SEGV} = local $SIG{HUP} = \&_trap,
-         local $SIG{QUIT} = \&_quit;
-         local $SIG{CHLD};
-
-         MCE::Shared::init() if $INC{'MCE/Shared.pm'};
-         $_DATA->{ $_SELF->{PKG} }->set('S'.$$, '') unless $self->{IGNORE};
-         CORE::kill($killed, $$) if $killed;
-
-         # Sets the seed of the base generator uniquely between workers.
-         # The new seed is computed using the current seed and ID value.
-         # One may set the seed at the application level for predictable
-         # results. Ditto for Math::Prime::Util, Math::Random, and
-         # Math::Random::MT::Auto.
-
-         srand( abs($_DATA->{"$pkg:seed"} - ($id * 100000)) % 2147483560 );
-
-         if ( $INC{'Math/Prime/Util.pm'} ) {
-            Math::Prime::Util::srand(
-                abs($_DATA->{"$pkg:seed"} - ($id * 100000)) % 2147483560
-            );
-         }
-
-         if ( $INC{'Math/Random.pm'} ) {
-            my $cur_seed = Math::Random::random_get_seed();
-            my $new_seed = ($cur_seed < 1073741781)
-               ? $cur_seed + ((abs($id) * 10000) % 1073741780)
-               : $cur_seed - ((abs($id) * 10000) % 1073741780);
-
-            Math::Random::random_set_seed($new_seed, $new_seed);
-         }
-
-         if ( $INC{'Math/Random/MT/Auto.pm'} ) {
-            my $cur_seed = Math::Random::MT::Auto::get_seed()->[0];
-            my $new_seed = ($cur_seed < 1073741781)
-               ? $cur_seed + ((abs($id) * 10000) % 1073741780)
-               : $cur_seed - ((abs($id) * 10000) % 1073741780);
-
-            Math::Random::MT::Auto::set_seed($new_seed);
-         }
-
-         _dispatch($mngd, $func, \@args);
-      }
+      return $self;
    }
 
-   $MCE::_GMUTEX->unlock() if ( $_tid && $MCE::_GMUTEX );
+   %{ $_LIST } = (), $_SELF = $self;                     # child
 
-   CORE::kill($killed, $$) if $killed;
+   if ( UNIVERSAL::can('Prima', 'cleanup') ) {
+      no warnings 'redefine'; local $@; eval '*Prima::cleanup = sub {}';
+   }
 
-   return $pid ? $self : undef;
+   MCE::Shared::init($id) if $INC{'MCE/Shared.pm'};
+
+   # Sets the seed of the base generator uniquely between workers.
+   # The new seed is computed using the current seed and ID value.
+   # One may set the seed at the application level for predictable
+   # results. Ditto for Math::Prime::Util, Math::Random, and
+   # Math::Random::MT::Auto.
+
+   srand( abs($_DATA->{"$pkg:seed"} - ($id * 100000)) % 2147483560 );
+
+   if ( $INC{'Math/Prime/Util.pm'} ) {
+      Math::Prime::Util::srand(
+          abs($_DATA->{"$pkg:seed"} - ($id * 100000)) % 2147483560
+      );
+   }
+
+   if ( $INC{'Math/Random.pm'} ) {
+      my $cur_seed = Math::Random::random_get_seed();
+      my $new_seed = ($cur_seed < 1073741781)
+         ? $cur_seed + ((abs($id) * 10000) % 1073741780)
+         : $cur_seed - ((abs($id) * 10000) % 1073741780);
+
+      Math::Random::random_set_seed($new_seed, $new_seed);
+   }
+
+   if ( $INC{'Math/Random/MT/Auto.pm'} ) {
+      my $cur_seed = Math::Random::MT::Auto::get_seed()->[0];
+      my $new_seed = ($cur_seed < 1073741781)
+         ? $cur_seed + ((abs($id) * 10000) % 1073741780)
+         : $cur_seed - ((abs($id) * 10000) % 1073741780);
+
+      Math::Random::MT::Auto::set_seed($new_seed);
+   }
+
+   _dispatch($mngd, $func, \@args);
 }
 
 ###############################################################################
@@ -286,7 +266,7 @@ sub equal {
 
 sub error {
    _croak('Usage: $child->error()') unless ref( my $self = $_[0] );
-   $self->join() unless ( exists $self->{JOINED} );
+   $self->join() if ( !exists $self->{JOINED} );
    $self->{ERROR} || undef;
 }
 
@@ -356,16 +336,16 @@ sub is_joinable {
       '';
    }
    elsif ( $self->{MGR_ID} eq "$$.$_tid" ) {
-      return '' if ( exists $self->{JOINED} );
-      local $!; $_DATA->{$pkg}->reap_data;
+      return undef if ( exists $self->{JOINED} );
+      local $!; $_DATA->{$pkg}->reapdata;
       ( waitpid($wrk_id, _WNOHANG) == 0 ) ? '' : do {
-         _reap_child($self, 0) unless ( exists $self->{JOINED} );
+         _reap_child($_LIST->{$pkg}->del($self->{WRK_ID}), 0);
          1;
       };
    }
    else {
-      # limitation for MCE::Child only; allowed for MCE::Hobo
-      _croak('Error: $child->is_joinable() not called by managed process');
+      return undef if ( exists $self->{JOINED} );
+      $_DATA->{$pkg}->exists('R'.$wrk_id) ? 1 : '';
    }
 }
 
@@ -377,28 +357,25 @@ sub is_running {
       1;
    }
    elsif ( $self->{MGR_ID} eq "$$.$_tid" ) {
-      return '' if ( exists $self->{JOINED} );
-      local $!; $_DATA->{$pkg}->reap_data;
+      return undef if ( exists $self->{JOINED} );
+      local $!; $_DATA->{$pkg}->reapdata;
       ( waitpid($wrk_id, _WNOHANG) == 0 ) ? 1 : do {
-         _reap_child($self, 0) unless ( exists $self->{JOINED} );
+         _reap_child($_LIST->{$pkg}->del($self->{WRK_ID}), 0);
          '';
       };
    }
    else {
-      # limitation for MCE::Child only; allowed for MCE::Hobo
-      _croak('Error: $child->is_running() not called by managed process');
+      return undef if ( exists $self->{JOINED} );
+      $_DATA->{$pkg}->exists('R'.$wrk_id) ? '' : 1;
    }
 }
 
 sub join {
    _croak('Usage: $child->join()') unless ref( my $self = $_[0] );
-   _croak('Cannot join a detached/ignored child') if $self->{IGNORE};
-
    my ( $wrk_id, $pkg ) = ( $self->{WRK_ID}, $self->{PKG} );
 
    if ( exists $self->{JOINED} ) {
       _croak('Child already joined') unless exists( $self->{RESULT} );
-      $_LIST->{$pkg}->del($wrk_id) if exists( $_LIST->{$pkg} );
 
       return ( defined wantarray )
          ? wantarray ? @{ delete $self->{RESULT} } : delete( $self->{RESULT} )->[-1]
@@ -409,17 +386,13 @@ sub join {
       _croak('Cannot join self');
    }
    elsif ( $self->{MGR_ID} eq "$$.$_tid" ) {
-      # remove from list after reaping
-      _reap_child($self, 1);
-      $_LIST->{$pkg}->del($wrk_id);
+      _reap_child($_LIST->{$pkg}->del($wrk_id), 1);
    }
    else {
-      # limitation for MCE::Child only; allowed for MCE::Hobo
-      _croak('Error: $child->join() not called by managed process');
+      sleep 0.3 until ( $_DATA->{$pkg}->exists('R'.$wrk_id) );
+      _reap_child($self, 0);
    }
 
-   return unless ( exists $self->{RESULT} );
-
    ( defined wantarray )
       ? wantarray ? @{ delete $self->{RESULT} } : delete( $self->{RESULT} )->[-1]
       : ();
@@ -454,13 +427,6 @@ sub list {
    ( exists $_LIST->{$pkg} ) ? $_LIST->{$pkg}->vals() : ();
 }
 
-sub list_pids {
-   _croak('Usage: MCE::Child->list_pids()') if ref($_[0]);
-   my $pkg = "$$.$_tid.".caller(); local $_;
-
-   ( exists $_LIST->{$pkg} ) ? map { $_->pid } $_LIST->{$pkg}->vals() : ();
-}
-
 sub list_joinable {
    _croak('Usage: MCE::Child->list_joinable()') if ref($_[0]);
    my $pkg = "$$.$_tid.".caller();
@@ -468,11 +434,11 @@ sub list_joinable {
    return () unless ( my $list = $_LIST->{$pkg} );
    local ($!, $?, $_);
 
-   $_DATA->{$pkg}->reap_data;
+   $_DATA->{$pkg}->reapdata;
 
    map {
       ( waitpid($_->{WRK_ID}, _WNOHANG) == 0 ) ? () : do {
-         _reap_child($_, 0) unless ( exists $_->{JOINED} );
+         _reap_child($list->del($_->{WRK_ID}), 0);
          $_;
       };
    }
@@ -486,11 +452,11 @@ sub list_running {
    return () unless ( my $list = $_LIST->{$pkg} );
    local ($!, $?, $_);
 
-   $_DATA->{$pkg}->reap_data;
+   $_DATA->{$pkg}->reapdata;
 
    map {
       ( waitpid($_->{WRK_ID}, _WNOHANG) == 0 ) ? $_ : do {
-         _reap_child($_, 0) unless ( exists $_->{JOINED} );
+         _reap_child($list->del($_->{WRK_ID}), 0);
          ();
       };
    }
@@ -531,7 +497,7 @@ sub pid {
 
 sub result {
    _croak('Usage: $child->result()') unless ref( my $self = $_[0] );
-   return $self->join() unless ( exists $self->{JOINED} );
+   return $self->join() if ( !exists $self->{JOINED} );
 
    _croak('Child already joined') unless exists( $self->{RESULT} );
    wantarray ? @{ delete $self->{RESULT} } : delete( $self->{RESULT} )->[-1];
@@ -570,19 +536,15 @@ sub wait_one {
 sub yield {
    _croak('Usage: MCE::Child->yield()') if ref($_[0]);
    shift if ( defined $_[0] && $_[0] eq __PACKAGE__ );
+   my $pkg = $_SELF->{PKG};
 
-   my $pkg = $_SELF->{PKG} || do {
-      my $mngd = $_MNGD->{ "$$.$_tid.".caller() } || do {
-         # construct mngd internally on first use unless defined
-         init(); $_MNGD->{ "$$.$_tid.".caller() };
-      };
-      $mngd->{PKG};
-   };
+   return unless ( my $mngd = $_MNGD->{$pkg} );
 
-   return unless $_DELY->{$pkg};
-   my $seconds = $_DELY->{$pkg}->seconds(@_);
+   ( $INC{'Coro/AnyEvent.pm'} )
+      ? Coro::AnyEvent::sleep( $_DELY->{$pkg}->seconds(@_) )
+      : sleep $_DELY->{$pkg}->seconds(@_);
 
-   MCE::Util::_sleep( $seconds );
+   return;
 }
 
 ###############################################################################
@@ -605,9 +567,11 @@ sub _croak {
 
 sub _dispatch {
    my ( $mngd, $func, $args ) = @_;
+   $mngd->{WRK_ID} = $_SELF->{WRK_ID} = $$;
 
-   $mngd->{WRK_ID} = $_SELF->{WRK_ID} = $$, $? = 0;
    $ENV{PERL_MCE_IPC} = 'win32' if $_is_MSWin32;
+   $SIG{TERM} = $SIG{INT} = $SIG{HUP} = \&_trap;
+   $SIG{QUIT} = \&_quit;
 
    {
       local $!;
@@ -616,6 +580,8 @@ sub _dispatch {
    }
 
    # Run task.
+   $_DATA->{ $_SELF->{PKG} }->set('S'.$$, ''), $? = 0;
+
    my $child_timeout = ( exists $_SELF->{child_timeout} )
       ? $_SELF->{child_timeout} : $mngd->{child_timeout};
 
@@ -639,24 +605,18 @@ sub _dispatch {
       };
    }
 
-   alarm 0;
+   alarm 0; _exit($?) if ( $@ && $@ =~ /^Child exited \(\S+\)$/ );
 
    if ( $@ ) {
-      _exit($?) if ( $@ =~ /^Child exited \(\S+\)$/ );
-      my $err = $@; $? = 1;
-
-      $_DATA->{ $_SELF->{PKG} }->set('S'.$$, $err),
-      $_DATA->{ $_SELF->{PKG} }->set('R'.$$, @res ? $_freeze->(\@res) : '')
-         if ( ! $_SELF->{IGNORE} );
-
-      warn "Child $$ terminated abnormally: reason $err\n"
-         if ( $err ne "Child timed out" && !$mngd->{on_finish} );
-   }
-   else {
-      $_DATA->{ $_SELF->{PKG} }->set('R'.$$, @res ? $_freeze->(\@res) : '')
-         if ( ! $_SELF->{IGNORE} );
+      my $err = $@;
+      $? = 1, $_DATA->{ $_SELF->{PKG} }->set('S'.$$, $err);
+      warn "Child $$ terminated abnormally: reason $err\n" if (
+         $err ne "Child timed out" && !$mngd->{on_finish}
+      );
    }
 
+   $_DATA->{ $_SELF->{PKG} }->set('R'.$$, @res ? $_freeze->(\@res) : '');
+
    _exit($?);
 }
 
@@ -664,18 +624,29 @@ sub _exit {
    my ( $exit_status ) = @_;
 
    # Check for nested workers not yet joined.
-   MCE::Child->finish('MCE') if ( !$_SELF->{SIGNALED} && keys %{ $_LIST } );
+   if ( !$_SELF->{SIGNALED} ) {
+      MCE::Child->finish('MCE') if ( keys %{ $_LIST } > 0 );
+      MCE::Hobo->finish('MCE')  if ( $INC{'MCE/Hobo.pm'} );
+   }
 
    # Exit child process.
-   $SIG{__DIE__}  = sub {} unless $_tid;
-   $SIG{__WARN__} = sub {};
+   $SIG{__DIE__}  = sub { } unless $_tid;
+   $SIG{__WARN__} = sub { };
 
-   threads->exit($exit_status) if ( $INC{'threads.pm'} && $_is_MSWin32 );
+   threads->exit($exit_status) if ( $_has_threads && $_is_MSWin32 );
+
+   if ( ! $_tid ) {
+      $SIG{HUP} = $SIG{INT} = $SIG{QUIT} = $SIG{TERM} = sub {
+         $SIG{$_[0]} = $SIG{INT} = sub { };
+         CORE::kill($_[0], getppid()) if ( $_[0] eq 'INT' && !$_is_MSWin32 );
+         CORE::kill('KILL', $$);
+      };
+   }
 
    my $posix_exit = ( exists $_SELF->{posix_exit} )
       ? $_SELF->{posix_exit} : $_MNGD->{ $_SELF->{PKG} }{posix_exit};
 
-   if ( ( $posix_exit || $_SELF->{SIGNALED} ) && !$_is_MSWin32 ) {
+   if ( $posix_exit && !$_is_MSWin32 ) {
       eval { MCE::Mutex::Channel::_destroy() };
       POSIX::_exit($exit_status) if $INC{'POSIX.pm'};
       CORE::kill('KILL', $$);
@@ -690,7 +661,7 @@ sub _force_reap {
 
    for my $child ( $_LIST->{$pkg}->vals() ) {
       if ( $child->is_running() ) {
-         sleep(0.015), CORE::kill('KILL', $child->pid())
+         CORE::kill('KILL', $child->pid())
             if CORE::kill('ZERO', $child->pid());
          $count++;
       }
@@ -705,8 +676,6 @@ sub _force_reap {
 }
 
 sub _quit {
-   return MCE::Signal::defer($_[0]) if $MCE::Signal::IPC;
-
    my ( $name ) = @_;
    $_SELF->{SIGNALED} = 1, $name =~ s/^SIG//;
 
@@ -718,8 +687,6 @@ sub _quit {
 
 sub _reap_child {
    my ( $child, $wait_flag ) = @_;
-   return unless $child;
-
    local @_ = $_DATA->{ $child->{PKG} }->get( $child->{WRK_ID}, $wait_flag );
 
    ( $child->{ERROR}, $child->{RESULT}, $child->{JOINED} ) =
@@ -732,7 +699,6 @@ sub _reap_child {
       if ( ( $code > 100 || $sig == 9 ) && !$err ) {
          $code = 2, $sig = 1,  $err = 'received SIGHUP'  if $code == 101;
          $code = 2, $sig = 2,  $err = 'received SIGINT'  if $code == 102;
-         $code = 2, $sig = 11, $err = 'received SIGSEGV' if $code == 111;
          $code = 2, $sig = 15, $err = 'received SIGTERM' if $code == 115;
          $code = 2, $sig = 9,  $err = 'received SIGKILL' if $sig  == 9;
       }
@@ -747,8 +713,6 @@ sub _reap_child {
 }
 
 sub _trap {
-   return MCE::Signal::defer($_[0]) if $MCE::Signal::IPC;
-
    my ( $exit_status, $name ) = ( 2, @_ );
    $_SELF->{SIGNALED} = 1, $name =~ s/^SIG//;
 
@@ -757,7 +721,6 @@ sub _trap {
 
    if    ( $name eq 'HUP'  ) { $exit_status = 101 }
    elsif ( $name eq 'INT'  ) { $exit_status = 102 }
-   elsif ( $name eq 'SEGV' ) { $exit_status = 111 }
    elsif ( $name eq 'TERM' ) { $exit_status = 115 }
 
    _exit($exit_status);
@@ -765,17 +728,19 @@ sub _trap {
 
 sub _wait_one {
    my ( $pkg ) = @_;
-   my ( $list, $self, $wrk_id ) = ( $_LIST->{$pkg} ); local $!;
+   my ( $list, $data ) = ( $_LIST->{$pkg}, $_DATA->{$pkg} );
+   my ( $self, $wrk_id, $found ); local $!;
 
    while () {
-      $_DATA->{$pkg}->reap_data;
       for my $child ( $list->vals ) {
          $wrk_id = $child->{WRK_ID};
-         return  $list->del($wrk_id) if ( exists $child->{JOINED} );
+         $found  = $data->exists('R'.$wrk_id);
+         waitpid($wrk_id, 0), $self = $list->del($wrk_id), last if $found;
+
          $self = $list->del($wrk_id), last if waitpid($wrk_id, _WNOHANG);
       }
       last if $self;
-      sleep 0.030;
+      sleep 0.015;
    }
 
    _reap_child($self, 0);
@@ -806,15 +771,12 @@ sub new {
 
 sub seconds {
    my ( $self, $how_long ) = @_;
-   my $delay = defined($how_long) ? $how_long : $self->[0];
-   my $lapse = $self->[1]->recv();
-   my $time  = MCE::Util::_time();
+   my ( $delay, $time ) = ( $how_long || $self->[0], Time::HiRes::time() );
+   my ( $lapse ) = $self->[1]->recv();
 
-   if ( !$delay || !defined $lapse ) {
-      $lapse = $time;
-   }
-   elsif ( $lapse + $delay - $time < 0 ) {
-      $lapse += int( abs($time - $lapse) / $delay + 0.5 ) * $delay;
+   if ( !$lapse || $time >= $lapse ) {
+      $self->[1]->send($time + $delay);
+      return $delay;
    }
 
    $self->[1]->send( $lapse += $delay );
@@ -865,12 +827,6 @@ sub exists {
 sub get {
    my ( $self, $wrk_id, $wait_flag ) = @_;
 
-   if ( !CORE::exists $self->[0]{ 'R'.$wrk_id } ) {
-      while ( my $data = $self->[1]->recv2_nb() ) {
-         $self->[0]{ $data->[0] } = $data->[1];
-      }
-   }
-
    if ( $wait_flag ) {
       local $!;
       ( CORE::exists $self->[0]{ 'R'.$wrk_id } ) ? waitpid($wrk_id, 0) : do {
@@ -878,29 +834,24 @@ sub get {
             my $data = $self->[1]->recv2_nb();
             if ( !defined $data ) {
                last if waitpid($wrk_id, _WNOHANG);
-               sleep(0.030), next;
+               sleep(0.015), next;
             }
             $self->[0]{ $data->[0] } = $data->[1];
             waitpid($wrk_id, 0), last if $data->[0] eq 'R'.$wrk_id;
          }
-         if ( !CORE::exists $self->[0]{ 'R'.$wrk_id } ) {
-            while ( my $data = $self->[1]->recv2_nb() ) {
-               $self->[0]{ $data->[0] } = $data->[1];
-            }
-         }
       };
    }
 
    my $result = delete $self->[0]{ 'R'.$wrk_id };
    my $error  = delete $self->[0]{ 'S'.$wrk_id };
 
-   $result = '' unless ( defined $result );
-   $error  = '' unless ( defined $error  );
+   $result = '' unless defined $result;
+   $error  = '' unless defined $error;
 
    return ( $result, $error );
 }
 
-sub reap_data {
+sub reapdata {
    my ( $self ) = @_;
 
    while ( my $data = $self->[1]->recv2_nb() ) {
@@ -932,7 +883,7 @@ sub clear {
 sub del {
    my ( $data, $keys, $indx, $gcnt ) = @{ $_[0] };
    my $pos = delete $indx->{ $_[1] };
-   return undef unless ( defined $pos );
+   return undef unless defined $pos;
 
    $keys->[ $pos ] = undef;
 
@@ -979,7 +930,7 @@ MCE::Child - A threads-like parallelization module compatible with Perl 5.8
 
 =head1 VERSION
 
-This document describes MCE::Child version 1.867_01
+This document describes MCE::Child version 1.839
 
 =head1 SYNOPSIS
 
@@ -1011,14 +962,12 @@ This document describes MCE::Child version 1.867_01
  MCE::Child->create( \&parallel, $_ ) for 1 .. 3;
 
  my @procs    = MCE::Child->list();
- my @pids     = MCE::Child->list_pids();
  my @running  = MCE::Child->list_running();
  my @joinable = MCE::Child->list_joinable();
  my @count    = MCE::Child->pending();
 
  # Joining is orderly, e.g. child1 is joined first, child2, child3.
- $_->join() for @procs;   # (or)
- $_->join() for @joinable;
+ $_->join() for @procs;
 
  # Joining occurs immediately as child processes complete execution.
  1 while MCE::Child->wait_one();
@@ -1164,7 +1113,7 @@ The C<new()> method is an alias for C<create()>.
 
 =item mce_child { BLOCK };
 
-C<mce_child> runs the block asynchronously similarly to C<< MCE::Child->create() >>.
+C<mce_child> runs the block asynchronously similarly to C<MCE::Child->create()>.
 It returns the child object, or undef if child creation failed.
 
  my $child = mce_child { foreach (@files) { ... } };
@@ -1233,10 +1182,9 @@ join later if not immediately to not leave a zombie or defunct process.
 
 =item MCE::Child->exit( 0, @ret )
 
-A child can exit at any time by calling C<< MCE::Child->exit() >>.
-Otherwise, the behavior is the same as C<exit(status)> when called from
-the main process. The child process may optionally return data, to be
-sent via IPC.
+A child can exit at any time by calling C<MCE::Child->exit()>. Otherwise, the
+behavior is the same as C<exit(status)> when called from the main process.
+The child process may optionally return data, to be sent via IPC.
 
 =item MCE::Child->finish()
 
@@ -1292,8 +1240,8 @@ to terminate after some time. The default is C<0> for no timeout.
 
 Set C<posix_exit> to avoid all END and destructor processing. Constructing
 MCE::Child inside a thread implies 1 or if present CGI, FCGI, Coro, Curses,
-Gearman::Util, Gearman::XS, LWP::UserAgent, Mojo::IOLoop, STFL, Tk, Wx,
-or Win32::GUI.
+Gearman::Util, Gearman::XS, LWP::UserAgent, Mojo::IOLoop, Prima, STFL,
+Tk, Wx, or Win32::GUI.
 
 Set C<void_context> to create the child process in void context for the
 return value. Otherwise, the return context is wantarray-aware for
@@ -1407,18 +1355,6 @@ Returns a list of all child objects not yet joined.
 
  @procs = MCE::Child->list();
 
-=item MCE::Child->list_pids()
-
-Returns a list of all child pids not yet joined (available since 1.849).
-
- @pids = MCE::Child->list_pids();
-
- $SIG{INT} = $SIG{HUP} = $SIG{TERM} = sub {
-     # Signal workers all at once
-     CORE::kill('KILL', MCE::Child->list_pids());
-     exec('reset');
- };
-
 =item MCE::Child->list_running()
 
 Returns a list of all child objects that are still running.
@@ -1514,19 +1450,14 @@ Class methods that allows a child to obtain its own ID.
 
 =item MCE::Child->wait_one()
 
-=item MCE::Child->waitone()
-
 =item MCE::Child->wait_all()
 
-=item MCE::Child->waitall()
-
 Meaningful for the manager process only, waits for one or all child processes
 to complete execution. Afterwards, returns the corresponding child objects.
 If a child doesn't exist, returns the C<undef> value or an empty list for
 C<wait_one> and C<wait_all> respectively.
 
-The C<waitone> and C<waitall> methods are aliases for compatibility with
-C<MCE::Hobo>.
+The C<waitone> and C<waitall> methods are aliases respectively.
 
  use MCE::Child;
  use Time::HiRes qw(sleep);
@@ -1571,9 +1502,6 @@ Give other workers a chance to run, optionally for given time. Yield behaves
 similarly to MCE's interval option. It throttles workers from running too fast.
 A demonstration is provided in the next section for fetching URLs in parallel.
 
-The default C<floating_seconds> is 0.008 and 0.015 on UNIX and Windows,
-respectively. Pass 0 if simply wanting to give other workers a chance to run.
-
  # total run time: 1.00 second
 
  MCE::Child->create( sub { MCE::Child->yield(0.25) } ) for 1 .. 4;
@@ -1581,150 +1509,6 @@ respectively. Pass 0 if simply wanting to give other workers a chance to run.
 
 =back
 
-=head1 THREADS-like DETACH CAPABILITY ON UNIX
-
-Threads-like detach capability was added starting with the 1.867 release.
-
-A threads example is shown first followed by the MCE::Child example. All one
-needs to do is set the CHLD signal handler to IGNORE. Unfortunately, this works
-on UNIX platforms only. Similarly to threads, the process is detached where one
-can no longer join it. The child process restores the CHLD handler to default,
-so is able to deeply spin workers and reap if desired.
-
- use threads;
-
- for ( 1 .. 8 ) {
-     async {
-         # do something
-     }->detach;
- }
-
- use MCE::Child;
-
- # Have the OS reap workers automatically.
- $SIG{CHLD} = 'IGNORE';
-
- for ( 1 .. 8 ) {
-     mce_child {
-         # do something
-     };
- }
-
-The following is another way although not detach-like for the same result
-and works on the Windows platform.
-
- use MCE::Child;
-
- for ( 1 .. 8 ) {
-     $_->join for MCE::Child->list_joinable;
-     mce_child {
-         # do something
-     };
- }
-
- MCE::Child->waitall;
-
-=head1 PARALLEL::FORKMANAGER-like DEMONSTRATION
-
-MCE::Child behaves similarly to threads for the most part. It also provides
-L<Parallel::ForkManager>-like capabilities. The C<Parallel::ForkManager>
-example is shown first followed by a version using C<MCE::Child>.
-
-=over 3
-
-=item Parallel::ForkManager
-
- use strict;
- use warnings;
-
- use Parallel::ForkManager;
- use Time::HiRes 'time';
-
- my $start = time;
-
- my $pm = Parallel::ForkManager->new(10);
- $pm->set_waitpid_blocking_sleep(0);
-
- $pm->run_on_finish( sub {
-     my ($pid, $exit_code, $ident, $exit_signal, $core_dumped, $resp) = @_;
-     print "child $pid completed: $ident => ", $resp->[0], "\n";
- });
-
- DATA_LOOP:
- foreach my $data ( 1..2000 ) {
-     # forks and returns the pid for the child
-     my $pid = $pm->start($data) and next DATA_LOOP;
-     my $ret = [ $data * 2 ];
-
-     $pm->finish(0, $ret);
- }
-
- $pm->wait_all_children;
-
- printf STDERR "duration: %0.03f seconds\n", time - $start;
-
-=item MCE::Child
-
- use strict;
- use warnings;
-
- use MCE::Child 1.843;
- use Time::HiRes 'time';
-
- my $start = time;
-
- MCE::Child->init(
-     max_workers => 10,
-     on_finish   => sub {
-         my ($pid, $exit_code, $ident, $exit_signal, $error, $resp) = @_;
-         print "child $pid completed: $ident => ", $resp->[0], "\n";
-     }
- );
-
- foreach my $data ( 1..2000 ) {
-     MCE::Child->create( $data, sub {
-         [ $data * 2 ];
-     });
- }
-
- MCE::Child->wait_all;
-
- printf STDERR "duration: %0.03f seconds\n", time - $start;
-
-=item Time to spin 2,000 workers and obtain results (in seconds).
-
-Results were obtained on a Macbook Pro (2.6 GHz ~ 3.6 GHz with Turbo Boost).
-Parallel::ForkManager 2.02 uses Moo. Therefore, I ran again with Moo loaded
-at the top of the script.
-
- MCE::Hobo uses MCE::Shared to retrieve data during reaping.
- MCE::Child uses MCE::Channel, no shared-manager.
-
-          Version  Cygwin   Windows  Linux   macOS  FreeBSD
-
- MCE::Child 1.843  19.099s  17.091s  0.965s  1.534s  1.229s
-  MCE::Hobo 1.843  20.514s  19.594s  1.246s  1.629s  1.613s
-      P::FM 1.20   19.703s  19.235s  0.875s  1.445s  1.346s
-
- MCE::Child 1.843  20.426s  18.417s  1.116s  1.632s  1.338s  Moo loaded
-  MCE::Hobo 1.843  21.809s  20.810s  1.407s  1.759s  1.722s  Moo loaded
-      P::FM 2.02   21.668s  25.927s  1.882s  2.612s  2.483s  Moo used
-
-=item Set posix_exit to avoid all END and destructor processing.
-
-This is helpful for reducing overhead when workers exit. Ditto if using a Perl
-module not parallel safe. The option is ignored on Windows C<$^O eq 'MSWin32'>.
-
- MCE::Child->init( posix_exit => 1, ... );
-  MCE::Hobo->init( posix_exit => 1, ... );
-
-          Version  Cygwin   Windows  Linux   macOS  FreeBSD
-
- MCE::Child 1.843  19.815s  ignored  0.824s  1.284s  1.245s  Moo loaded
-  MCE::Hobo 1.843  21.029s  ignored  0.953s  1.335s  1.439s  Moo loaded
-
-=back
-
 =head1 PARALLEL HTTP GET DEMONSTRATION USING ANYEVENT
 
 This demonstration constructs two queues, two handles, starts the
@@ -1995,14 +1779,6 @@ threads are necessary for the binary to exit successfully.
 
  threads->exit(0) if $INC{"threads.pm"};
 
-=head1 LIMITATION
-
-MCE::Child emits an error when C<is_joinable>, C<is_running>, and C<join> isn't
-called by the managed process, where the child was spawned. This is a limitation
-in MCE::Child only due to not involving a shared-manager process for IPC.
-
-This use-case is not typical.
-
 =head1 CREDITS
 
 The inspiration for C<MCE::Child> comes from wanting C<threads>-like behavior
