@@ -14,7 +14,7 @@ package MCE::Core::Input::Iterator;
 use strict;
 use warnings;
 
-our $VERSION = '1.838';
+our $VERSION = '1.839';
 
 ## Items below are folded into MCE.
 
@@ -59,7 +59,7 @@ sub _worker_user_iterator {
             unless $_DAT_LOCK->{ $_pid };
       };
       $_dat_un = sub {
-         MCE::Util::_syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+         syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
             if $_DAT_LOCK->{ $_pid };
       };
    }
