@@ -14,7 +14,7 @@ package MCE::Core::Input::Sequence;
 use strict;
 use warnings;
 
-our $VERSION = '1.835';
+our $VERSION = '1.839';
 
 ## Items below are folded into MCE.
 
@@ -56,11 +56,11 @@ sub _worker_sequence_queue {
       $_pid = $INC{'threads.pm'} ? $$ .'.'. threads->tid() : $$;
 
       # inlined for performance
-      if ($self->{_data_channels} > 6) {
-         $_DAT_LOCK = $self->{'_mutex_'.( $self->{_wid} % 6 + 1 )};
+      if ($self->{_data_channels} > 5) {
+         $_DAT_LOCK = $self->{'_mutex_'.( $self->{_wid} % 5 + 1 )};
       }
       $_dat_ex = sub {
-         sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
+         MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
             unless $_DAT_LOCK->{ $_pid };
       };
       $_dat_un = sub {
@@ -97,23 +97,19 @@ sub _worker_sequence_queue {
 
       ## Obtain the next chunk_id and sequence number.
       $_dat_ex->() if $_lock_chn;
-
-      1 until sysread($_QUE_R_SOCK, $_next, $_que_read_size) || ($! && !$!{'EINTR'});
+      MCE::Util::_sysread($_QUE_R_SOCK, $_next, $_que_read_size);
 
       ($_chunk_id, $_offset) = unpack($_que_template, $_next);
 
       if ($_offset >= $_abort) {
-         1 until syswrite (
-            $_QUE_W_SOCK, pack($_que_template, 0, $_offset)
-         ) || ($! && !$!{'EINTR'});
-
+         syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_offset));
          $_dat_un->() if $_lock_chn;
          return;
       }
 
-      1 until syswrite (
+      syswrite(
          $_QUE_W_SOCK, pack($_que_template, $_chunk_id + 1, $_offset + 1)
-      ) || ($! && !$!{'EINTR'});
+      );
 
       $_dat_un->() if $_lock_chn;
       $_chunk_id++;
