@@ -14,7 +14,7 @@ package MCE::Core::Manager;
 use strict;
 use warnings;
 
-our $VERSION = '1.835';
+our $VERSION = '1.839';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -88,7 +88,7 @@ sub _output_loop {
       $_has_user_tasks, $_sess_dir, $_task_id, $_user_error, $_user_output,
       $_input_size, $_offset_pos, $_single_dim, @_gather, $_cs_one_flag,
       $_exit_id, $_exit_pid, $_exit_status, $_exit_wid, $_len, $_sync_cnt,
-      $_BSB_W_SOCK, $_BSE_W_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
+      $_BSB_W_SOCK, $_BSB_R_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
       $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT,
       @_delay_wid, $_size_completed, $_win32_ipc
    );
@@ -116,7 +116,7 @@ sub _output_loop {
       my $_ret = $_cb->(@_);
 
       return print {$_DAU_R_SOCK} length($_ret).'0'.$LF, $_ret
-         if ( !ref $_ret && defined $_ret && !looks_like_number $_ret );
+         if ( !looks_like_number $_ret && !ref $_ret && defined $_ret );
 
       my $_buf = $self->{freeze}([ $_ret ]);
 
@@ -148,7 +148,7 @@ sub _output_loop {
          if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
             if ($_sync_cnt == $_total_running) {
                for my $_i (1 .. $_total_running) {
-                  1 until syswrite($_BSB_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+                  syswrite($_BSB_W_SOCK, $LF);
                }
                undef $_syn_flag;
             }
@@ -180,7 +180,7 @@ sub _output_loop {
          if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
             if ($_sync_cnt == $_total_running) {
                for my $_i (1 .. $_total_running) {
-                  1 until syswrite($_BSB_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+                  syswrite($_BSB_W_SOCK, $LF);
                }
                undef $_syn_flag;
             }
@@ -547,11 +547,9 @@ sub _output_loop {
 
             binmode $_sendto_fhs{$_file};
 
-            ## Select new FH, turn on autoflush, restore the old FH.
             if ($_flush_file) {
                local $!;
-               # IO::Handle->autoflush not available in older Perl.
-               select(( select($_sendto_fhs{$_file}), $| = 1 )[0]);
+               $_sendto_fhs{$_file}->autoflush(1);
             }
          }
 
@@ -576,11 +574,9 @@ sub _output_loop {
 
             binmode $_sendto_fhs{$_fd};
 
-            ## Select new FH, turn on autoflush, restore the old FH.
             if ($_flush_file) {
                local $!;
-               # IO::Handle->autoflush not available in older Perl.
-               select(( select($_sendto_fhs{$_fd}), $| = 1 )[0]);
+               $_sendto_fhs{$_fd}->autoflush(1);
             }
          }
 
@@ -603,7 +599,7 @@ sub _output_loop {
 
          if (++$_sync_cnt == $_total_running) {
             for my $_i (1 .. $_total_running) {
-               1 until syswrite($_BSB_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+               syswrite($_BSB_W_SOCK, $LF);
             }
             undef $_syn_flag;
          }
@@ -618,7 +614,7 @@ sub _output_loop {
                : $self->{_total_running};
 
             for my $_i (1 .. $_total_running) {
-               1 until syswrite($_BSE_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+               syswrite($_BSB_R_SOCK, $LF);
             }
          }
 
@@ -626,7 +622,7 @@ sub _output_loop {
       },
 
       OUTPUT_S_IPC.$LF => sub {                   # Change to win32 IPC
-         1 until syswrite($_DAT_R_SOCK, $LF) || ($! && !$!{'EINTR'});
+         syswrite($_DAT_R_SOCK, $LF);
 
          $_win32_ipc = 1, goto _LOOP unless $_win32_ipc;
 
@@ -794,28 +790,28 @@ sub _output_loop {
    }
 
    ## Autoflush STDERR-STDOUT handles if requested.
-   ## Make MCE_STDOUT the default handle.
-
-   my $_old_hndl = select $_MCE_STDOUT;
 
    {
       local $!;
-      # IO::Handle->autoflush not available in older Perl.
-      select($_MCE_STDERR), $| = 1 if ($self->{flush_stderr});
-      select($_MCE_STDOUT), $| = 1 if ($self->{flush_stdout});
-      select($_MCE_STDOUT);
+      $_MCE_STDERR->autoflush(1) if $self->{flush_stderr};
+      $_MCE_STDOUT->autoflush(1) if $self->{flush_stdout};
    }
 
    ## -------------------------------------------------------------------------
 
    ## Output event loop.
 
-   my $_func; my $_channels = $self->{_dat_r_sock};
+   my $_channels = $self->{_dat_r_sock};
+   my $_func;
 
-   $_win32_ipc  = ( $ENV{'PERL_MCE_IPC'} eq 'win32' || $INC{'MCE/Hobo.pm'} );
+   $_win32_ipc = (
+      $ENV{'PERL_MCE_IPC'} eq 'win32' ||
+      $INC{'MCE/Child.pm'} ||
+      $INC{'MCE/Hobo.pm'}
+   );
 
    $_BSB_W_SOCK = $self->{_bsb_w_sock};
-   $_BSE_W_SOCK = $self->{_bse_w_sock};
+   $_BSB_R_SOCK = $self->{_bsb_r_sock};
    $_DAT_R_SOCK = $self->{_dat_r_sock}->[0];
 
    $_RS     = $self->{RS} || $/;
@@ -874,50 +870,11 @@ sub _output_loop {
 
    ## Wait on requests *without* timeout capability.
 
-   elsif ($^O eq 'MSWin32' && $_win32_ipc) {
-      # The normal loop hangs on Windows when processes/threads start/exit.
-      # Using ioctl() properly, http://www.perlmonks.org/?node_id=780083
-
-      my $_val_bytes = "\x00\x00\x00\x00";
-      my $_ptr_bytes = unpack( 'I', pack('P', $_val_bytes) );
-      my ($_done, $_count, $_nbytes, $_start) = (0);
-
-      while (!$_done) {
-         $_start = time, $_count = 1;
-
-         # MSWin32 FIONREAD
-         IOCTL: ioctl($_DAT_R_SOCK, 0x4004667f, $_ptr_bytes);
-
-         unless ($_nbytes = unpack('I', $_val_bytes)) {
-            if ($_count) {
-                # delay after a while to not consume a CPU core
-                $_count = 0 if ++$_count % 50 == 0 && time - $_start > 0.030;
-            } else {
-                sleep 0.030;
-            }
-            goto IOCTL;
-         }
-
-         do {
-            sysread($_DAT_R_SOCK, $_func, 8);
-            $_done = 1, last() unless length($_func) == 8;
-            $_DAU_R_SOCK = $_channels->[ substr($_func, -2, 2, '') ];
-
-            if (exists $_core_output_function{$_func}) {
-               $_core_output_function{$_func}();
-            } elsif (exists $_plugin_function->{$_func}) {
-               $_plugin_function->{$_func}();
-            }
-
-         } while (($_nbytes -= 8) >= 8);
-
-         last unless $self->{_total_running};
-      }
-   }
-
    elsif ($^O eq 'MSWin32') {
+      MCE::Util::_nonblocking($_DAT_R_SOCK, 1) if $_win32_ipc;
+
       while ($self->{_total_running}) {
-         sysread($_DAT_R_SOCK, $_func, 8);
+         MCE::Util::_sysread($_DAT_R_SOCK, $_func, 8);
          last() unless length($_func) == 8;
          $_DAU_R_SOCK = $_channels->[ substr($_func, -2, 2, '') ];
 
@@ -928,7 +885,6 @@ sub _output_loop {
          }
       }
    }
-
    else {
       while ($self->{_total_running}) {
          $_func = <$_DAT_R_SOCK>;
@@ -961,9 +917,7 @@ sub _output_loop {
       delete $_sendto_fhs{$_p};
    }
 
-   ## Restore the default handle. Close MCE STDOUT/STDERR handles.
-
-   select $_old_hndl;
+   ## Close MCE STDOUT/STDERR handles.
 
    eval q{
       close $_MCE_STDOUT if (fileno $_MCE_STDOUT > 2);
