@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.835';
+our $VERSION = '1.839';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -75,10 +75,10 @@ sub import {
       _croak("Error: ($_argument) invalid module option");
    }
 
-   $_p->{MAX_WORKERS} = MCE::Util::_parse_max_workers($_p->{MAX_WORKERS});
+   $_p->{MAX_WORKERS} = MCE::_parse_max_workers($_p->{MAX_WORKERS});
 
-   _validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS');
-   _validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE')
+   MCE::_validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS', $_tag);
+   MCE::_validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE', $_tag)
       unless ($_p->{CHUNK_SIZE} eq 'auto');
 
    return;
@@ -258,7 +258,7 @@ sub run (&@) {
    }
 
    if (defined (my $_p = $_params->{$_pid})) {
-      $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
+      $_max_workers = MCE::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers});
 
       delete $_p->{sequence}    if (defined $_input_data || scalar @_);
@@ -269,7 +269,7 @@ sub run (&@) {
       delete $_p->{gather}      if (exists $_p->{gather});
    }
 
-   my $_chunk_size = MCE::Util::_parse_chunk_size(
+   my $_chunk_size = MCE::_parse_chunk_size(
       $_def->{$_pkg}{CHUNK_SIZE}, $_max_workers, $_params->{$_pid},
       $_input_data, scalar @_
    );
@@ -419,21 +419,6 @@ sub _croak {
    goto &MCE::_croak;
 }
 
-sub _validate_number {
-
-   my ($_n, $_key) = @_;
-
-   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
-
-   $_n =~ s/K\z//i; $_n =~ s/M\z//i;
-
-   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
-      _croak("$_tag: ($_key) is not valid");
-   }
-
-   return;
-}
-
 1;
 
 __END__
@@ -450,7 +435,7 @@ MCE::Map - Parallel map model similar to the native map function
 
 =head1 VERSION
 
-This document describes MCE::Map version 1.835
+This document describes MCE::Map version 1.839
 
 =head1 SYNOPSIS
 
@@ -459,18 +444,22 @@ This document describes MCE::Map version 1.835
 
  ## Array or array_ref
  my @a = mce_map { $_ * $_ } 1..10000;
- my @b = mce_map { $_ * $_ } [ 1..10000 ];
+ my @b = mce_map { $_ * $_ } \@list;
+
+ ## Important; pass an array_ref for deeply input data
+ my @c = mce_map { $_->[1] *= 2; $_ } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ my @d = mce_map { $_->[1] *= 2; $_ } \@deeply_list;
 
  ## File_path, glob_ref, or scalar_ref
- my @c = mce_map_f { chomp; $_ } "/path/to/file";
- my @d = mce_map_f { chomp; $_ } $file_handle;
- my @e = mce_map_f { chomp; $_ } \$scalar;
+ my @e = mce_map_f { chomp; $_ } "/path/to/file";
+ my @f = mce_map_f { chomp; $_ } $file_handle;
+ my @g = mce_map_f { chomp; $_ } \$scalar;
 
  ## Sequence of numbers (begin, end [, step, format])
- my @f = mce_map_s { $_ * $_ } 1, 10000, 5;
- my @g = mce_map_s { $_ * $_ } [ 1, 10000, 5 ];
+ my @h = mce_map_s { $_ * $_ } 1, 10000, 5;
+ my @i = mce_map_s { $_ * $_ } [ 1, 10000, 5 ];
 
- my @h = mce_map_s { $_ * $_ } {
+ my @j = mce_map_s { $_ * $_ } {
     begin => 1, end => 10000, step => 5, format => undef
  };
 
@@ -554,6 +543,8 @@ Specify C<Sereal => 0> to use Storable instead.
 
 =item MCE::Map::init { options }
 
+=back
+
 The init function accepts a hash of MCE options. The gather option, if
 specified, is ignored due to being used internally by the module.
 
@@ -595,8 +586,6 @@ specified, is ignored due to being used internally by the module.
  7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
  10000
 
-=back
-
 =head1 API DOCUMENTATION
 
 =over 3
@@ -605,18 +594,30 @@ specified, is ignored due to being used internally by the module.
 
 =item mce_map { code } list
 
+=back
+
 Input data may be defined using a list or an array reference. Unlike MCE::Loop,
 Flow, and Step, specifying a hash reference as input data isn't allowed.
 
+ ## Array or array_ref
  my @a = mce_map { $_ * 2 } 1..1000;
  my @b = mce_map { $_ * 2 } \@list;
 
- my @z = mce_map { $_ * 2 } \%hash;  # not supported
+ ## Important; pass an array_ref for deeply input data
+ my @c = mce_map { $_->[1] *= 2; $_ } [ [ 0, 1 ], [ 0, 2 ], ... ];
+ my @d = mce_map { $_->[1] *= 2; $_ } \@deeply_list;
+
+ ## Not supported
+ my @z = mce_map { ... } \%hash;
+
+=over 3
 
 =item MCE::Map->run_file ( sub { code }, file )
 
 =item mce_map_f { code } file
 
+=back
+
 The fastest of these is the /path/to/file. Workers communicate the next offset
 position among themselves with zero interaction by the manager process.
 
@@ -624,10 +625,14 @@ position among themselves with zero interaction by the manager process.
  my @d = mce_map_f { chomp; $_ . "\r\n" } $file_handle;
  my @e = mce_map_f { chomp; $_ . "\r\n" } \$scalar;
 
+=over 3
+
 =item MCE::Map->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
 
 =item mce_map_s { code } $beg, $end [, $step, $fmt ]
 
+=back
+
 Sequence may be defined as a list, an array reference, or a hash reference.
 The functions require both begin and end values to run. Step and format are
 optional. The format is passed to sprintf (% may be omitted below).
@@ -642,17 +647,19 @@ optional. The format is passed to sprintf (% may be omitted below).
     step => $step, format => $fmt
  };
 
+=over 3
+
 =item MCE::Map->run ( sub { code }, iterator )
 
 =item mce_map { code } iterator
 
+=back
+
 An iterator reference may be specified for input_data. Iterators are described
 under section "SYNTAX for INPUT_DATA" at L<MCE::Core>.
 
  my @a = mce_map { $_ * 2 } make_iterator(10, 30, 2);
 
-=back
-
 =head1 MANUAL SHUTDOWN
 
 =over 3
@@ -661,6 +668,8 @@ under section "SYNTAX for INPUT_DATA" at L<MCE::Core>.
 
 =item MCE::Map::finish
 
+=back
+
 Workers remain persistent as much as possible after running. Shutdown occurs
 automatically when the script terminates. Call finish when workers are no
 longer needed.
@@ -675,8 +684,6 @@ longer needed.
 
  MCE::Map::finish;
 
-=back
-
 =head1 INDEX
 
 L<MCE|MCE>, L<MCE::Core>
