@@ -11,30 +11,27 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.836';
+our $VERSION = '1.839';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use Carp ();
 
-use MCE::Mutex::Channel ();
-
 sub new {
     my ($class, %argv) = @_;
+    my $impl = defined($argv{'impl'})
+        ? $argv{'impl'} : defined($argv{'path'}) ? 'Flock' : 'Channel';
 
-    my $pkg = (defined $argv{'impl'})
-        ? $argv{'impl'} : (defined $argv{'path'}) ? 'Flock' : 'Channel';
+    $impl = ucfirst( lc $impl );
 
-    $pkg = ucfirst( lc $pkg );
+    eval "require MCE::Mutex::$impl; 1" ||
+        Carp::croak("Could not load Mutex implementation '$impl': $@");
 
-    if ($INC{"MCE/Mutex/$pkg.pm"} || eval "require MCE::Mutex::$pkg; 1") {
-        no strict 'refs'; $pkg = 'MCE::Mutex::'.$pkg;
+    my $pkg = 'MCE::Mutex::'.$impl;
+    no strict 'refs';
 
-        return $pkg->new(%argv);
-    }
-
-    Carp::croak("Could not load Mutex implementation $pkg: $@");
+    return $pkg->new( %argv );
 }
 
 ## base class methods
@@ -71,7 +68,7 @@ MCE::Mutex - Locking for Many-Core Engine
 
 =head1 VERSION
 
-This document describes MCE::Mutex version 1.836
+This document describes MCE::Mutex version 1.839
 
 =head1 SYNOPSIS
 
