@@ -11,15 +11,16 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized once );
 
-our $VERSION = '1.867_01';
+our $VERSION = '1.839';
 
 use base 'MCE::Mutex::Channel';
 use MCE::Util ();
 
-my $tid = $INC{'threads.pm'} ? threads->tid() : 0;
+my $has_threads = $INC{'threads.pm'} ? 1 : 0;
+my $tid = $has_threads ? threads->tid()  : 0;
 
 sub CLONE {
-    $tid = threads->tid() if $INC{'threads.pm'};
+    $tid = threads->tid() if $has_threads;
 }
 
 ###############################################################################
@@ -30,9 +31,9 @@ sub CLONE {
 
 sub new {
     my ($class, %obj) = (@_, impl => 'Channel2');
-    $obj{'_init_pid'} = $tid ? $$ .'.'. $tid : $$;
+    $obj{'_init_pid'} = $has_threads ? $$ .'.'. $tid : $$;
 
-    MCE::Util::_sock_pair(\%obj, qw(_r_sock _w_sock), undef, 1);
+    MCE::Util::_sock_pair(\%obj, qw(_r_sock _w_sock));
 
     syswrite $obj{_r_sock}, '0';
     syswrite $obj{_w_sock}, '0';
@@ -40,14 +41,14 @@ sub new {
     bless \%obj, $class;
 
     if ( caller !~ /^MCE:?/ || caller(1) !~ /^MCE:?/ ) {
-        MCE::Mutex::Channel::_save_for_global_cleanup(\%obj);
+        MCE::Mutex::Channel::_save_for_global_destruction(\%obj);
     }
 
     return \%obj;
 }
 
 sub lock2 {
-    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
 
     MCE::Util::_sysread($obj->{_w_sock}, my($b), 1), $obj->{ $pid.'b' } = 1
         unless $obj->{ $pid.'b' };
@@ -59,7 +60,7 @@ sub lock2 {
 *lock_shared2    = \&lock2;
 
 sub unlock2 {
-    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
 
     syswrite($obj->{_r_sock}, '0'), $obj->{ $pid.'b' } = 0
         if $obj->{ $pid.'b' };
@@ -68,7 +69,9 @@ sub unlock2 {
 }
 
 sub synchronize2 {
-    my ($pid, $obj, $code, @ret) = ($tid ? $$ .'.'. $tid : $$, shift, shift);
+    my ($pid, $obj, $code, @ret) = (
+        $has_threads ? $$ .'.'. $tid : $$, shift, shift
+    );
     return unless ref($code) eq 'CODE';
 
     # lock, run, unlock - inlined for performance
@@ -114,42 +117,26 @@ MCE::Mutex::Channel2 - Provides two mutexes using a single channel
 
 =head1 VERSION
 
-This document describes MCE::Mutex::Channel2 version 1.867_01
+This document describes MCE::Mutex::Channel2 version 1.839
 
 =head1 DESCRIPTION
 
-A socket implementation based on C<MCE::Mutex>. The secondary lock is accessed
-by calling methods with the C<2> suffix.
-
-The API is described in L<MCE::Mutex>.
-
-=head2 construction
-
-=over 3
-
-=item new
+A socket implementation based on L<MCE::Mutex>. The secondary lock is accessed
+by calling methods, described in L<MCE::Mutex>, with the C<2> suffix.
 
  my $mutex = MCE::Mutex->new( impl => 'Channel2' );
 
-=back
-
 =head2 primary lock
 
 =over 3
 
-=item lock
+=item * $mutex->lock
 
-=item lock_exclusive
+=item * $mutex->unlock
 
-=item lock_shared
+=item * $mutex->synchronize
 
-=item unlock
-
-=item synchronize
-
-=item enter
-
-=item timedwait
+=item * $mutex->timedwait
 
 =back
 
@@ -157,19 +144,13 @@ The API is described in L<MCE::Mutex>.
 
 =over 3
 
-=item lock2
+=item * $mutex->lock2
 
-=item lock_exclusive2
+=item * $mutex->unlock2
 
-=item lock_shared2
+=item * $mutex->synchronize2
 
-=item unlock2
-
-=item synchronize2
-
-=item enter2
-
-=item timedwait2
+=item * $mutex->timedwait2
 
 =back
 
